@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2012-2017 Oracle Corporation
+ * Copyright (C) 2012-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -23,8 +23,11 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-#ifndef ___VBox_RemoteDesktop_VRDEVideoIn_h
-#define ___VBox_RemoteDesktop_VRDEVideoIn_h
+#ifndef VBOX_INCLUDED_RemoteDesktop_VRDEVideoIn_h
+#define VBOX_INCLUDED_RemoteDesktop_VRDEVideoIn_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 
 /* Define VRDE_VIDEOIN_WITH_VRDEINTERFACE to include the server VRDE interface parts. */
@@ -1077,4 +1080,4 @@ typedef struct VRDEVIDEOINCALLBACKS
 
 #pragma pack()
 
-#endif
+#endif /* !VBOX_INCLUDED_RemoteDesktop_VRDEVideoIn_h */
