@@ -17,7 +17,7 @@
  */
 
 /*
- * Copyright (C) 2007-2016 Oracle Corporation
+ * Copyright (C) 2007-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -37,8 +37,11 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-#ifndef ___VBox_settings_h
-#define ___VBox_settings_h
+#ifndef VBOX_INCLUDED_settings_h
+#define VBOX_INCLUDED_settings_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/time.h>
 
@@ -232,13 +235,17 @@ protected:
     typedef enum {Error, HardDisk, DVDImage, FloppyImage} MediaType;
 
     static const char *stringifyMediaType(MediaType t);
-    SettingsVersion_T parseVersion(const com::Utf8Str &strVersion);
+    SettingsVersion_T parseVersion(const com::Utf8Str &strVersion,
+                                   const xml::ElementNode *pElm);
     void parseUUID(com::Guid &guid,
-                   const com::Utf8Str &strUUID) const;
+                   const com::Utf8Str &strUUID,
+                   const xml::ElementNode *pElm) const;
     void parseTimestamp(RTTIMESPEC &timestamp,
-                        const com::Utf8Str &str) const;
+                        const com::Utf8Str &str,
+                        const xml::ElementNode *pElm) const;
     void parseBase64(IconBlob &binary,
-                     const com::Utf8Str &str) const;
+                     const com::Utf8Str &str,
+                     const xml::ElementNode *pElm) const;
     com::Utf8Str stringifyTimestamp(const RTTIMESPEC &tm) const;
     void toBase64(com::Utf8Str &str,
                   const IconBlob &binary) const;
@@ -254,6 +261,7 @@ protected:
     void readNATLoopbacks(const xml::ElementNode &elmParent, NATLoopbackOffsetList &llLoopBacks);
 
     void setVersionAttribute(xml::ElementNode &elm);
+    void specialBackupIfFirstBump();
     void createStubDocument();
 
     void buildExtraData(xml::ElementNode &elmParent, const StringsMap &me);
@@ -312,7 +320,9 @@ struct SystemProperties
     com::Utf8Str            strDefaultAdditionsISO;
     com::Utf8Str            strDefaultFrontend;
     com::Utf8Str            strLoggingLevel;
-    uint32_t                ulLogHistoryCount;
+    com::Utf8Str            strProxyUrl;
+    uint32_t                uProxyMode; /**< ProxyMode_T */
+    uint32_t                uLogHistoryCount;
     bool                    fExclusiveHwVirt;
 };
 
@@ -416,6 +426,7 @@ private:
     void bumpSettingsVersionIfNeeded();
     void buildUSBDeviceSources(xml::ElementNode &elmParent, const USBDeviceSourcesList &ll);
     void readUSBDeviceSources(const xml::ElementNode &elmDeviceSources, USBDeviceSourcesList &ll);
+    bool convertGuiProxySettings(const com::Utf8Str &strUIProxySettings);
 };
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -472,6 +483,121 @@ struct BIOSSettings
     com::Utf8Str    strLogoImagePath;
 };
 
+/** List for keeping a recording feature list. */
+typedef std::map<RecordingFeature_T, bool> RecordingFeatureMap;
+
+struct RecordingScreenSettings
+{
+    RecordingScreenSettings();
+
+    virtual ~RecordingScreenSettings();
+
+    void applyDefaults(void);
+
+    bool areDefaultSettings(void) const;
+
+    bool isFeatureEnabled(RecordingFeature_T enmFeature) const;
+
+    bool operator==(const RecordingScreenSettings &d) const;
+
+    /** Whether to record this screen or not. */
+    bool                   fEnabled;   // requires settings version 1.14 (VirtualBox 4.3)
+    /** Destination to record to. */
+    RecordingDestination_T enmDest;    /** @todo Implement with next settings version bump. */
+    /** Which features are enable or not. */
+    RecordingFeatureMap    featureMap; /** @todo Implement with next settings version bump. */
+    /** Maximum time (in s) to record. If set to 0, no time limit is set. */
+    uint32_t               ulMaxTimeS; // requires settings version 1.14 (VirtualBox 4.3)
+    /** Options string for hidden / advanced / experimental features. */
+    com::Utf8Str           strOptions; // new since VirtualBox 5.2.
+
+    /**
+     * Structure holding settings for audio recording.
+     */
+    struct Audio
+    {
+        Audio()
+            : enmAudioCodec(RecordingAudioCodec_Opus)
+            , uHz(22050)
+            , cBits(16)
+            , cChannels(2) { }
+
+        /** The audio codec type to use. */
+        RecordingAudioCodec_T enmAudioCodec; /** @todo Implement with next settings version bump. */
+        /** Hz rate. */
+        uint16_t              uHz;           /** @todo Implement with next settings version bump. */
+        /** Bits per sample. */
+        uint8_t               cBits;         /** @todo Implement with next settings version bump. */
+        /** Number of audio channels. */
+        uint8_t               cChannels;     /** @todo Implement with next settings version bump. */
+    } Audio;
+
+    /**
+     * Structure holding settings for video recording.
+     */
+    struct Video
+    {
+        Video()
+            : enmCodec(RecordingVideoCodec_VP8)
+            , ulWidth(1024)
+            , ulHeight(768)
+            , ulRate(512)
+            , ulFPS(25) { }
+
+        /** The codec to use. */
+        RecordingVideoCodec_T enmCodec;  /** @todo Implement with next settings version bump. */
+        /** Target frame width in pixels (X). */
+        uint32_t              ulWidth;   // requires settings version 1.14 (VirtualBox 4.3)
+        /** Target frame height in pixels (Y). */
+        uint32_t              ulHeight;  // requires settings version 1.14 (VirtualBox 4.3)
+        /** Encoding rate. */
+        uint32_t              ulRate;    // requires settings version 1.14 (VirtualBox 4.3)
+        /** Frames per second (FPS). */
+        uint32_t              ulFPS;     // requires settings version 1.14 (VirtualBox 4.3)
+    } Video;
+
+    /**
+     * Structure holding settings if the destination is a file.
+     */
+    struct File
+    {
+        File()
+            : ulMaxSizeMB(0) { }
+
+        /** Maximum size (in MB) the file is allowed to have.
+         *  When reaching the limit, recording will stop. */
+        uint32_t     ulMaxSizeMB; // requires settings version 1.14 (VirtualBox 4.3)
+        /** Absolute file name path to use for recording. */
+        com::Utf8Str strName;     // requires settings version 1.14 (VirtualBox 4.3)
+    } File;
+};
+
+/** Map for keeping settings per virtual screen.
+ *  The key specifies the screen ID. */
+typedef std::map<uint32_t, RecordingScreenSettings> RecordingScreenMap;
+
+/**
+ * NOTE: If you add any fields in here, you must update a) the constructor and b)
+ * the operator== which is used by MachineConfigFile::operator==(), or otherwise
+ * your settings might never get saved.
+ */
+struct RecordingSettings
+{
+    RecordingSettings();
+
+    void applyDefaults(void);
+
+    bool areDefaultSettings(void) const;
+
+    bool operator==(const RecordingSettings &d) const;
+
+    /** Whether recording as a whole is enabled or disabled. */
+    bool               fEnabled;       // requires settings version 1.14 (VirtualBox 4.3)
+    /** Map of handled recording screen settings.
+     *  The key specifies the screen ID. */
+    RecordingScreenMap mapScreens;
+};
+
 /**
  * NOTE: If you add any fields in here, you must update a) the constructor and b)
  * the operator== which is used by MachineConfigFile::operator==(), or otherwise
@@ -590,6 +716,7 @@ struct SerialPort
     PortMode_T      portMode;
     com::Utf8Str    strPath;
     bool            fServer;
+    UartType_T      uartType;
 };
 
 typedef std::list<SerialPort> SerialPortsList;
@@ -629,6 +756,8 @@ struct AudioAdapter
     bool operator==(const AudioAdapter &a) const;
 
     bool                    fEnabled;
+    bool                    fEnabledIn;
+    bool                    fEnabledOut;
     AudioControllerType_T   controllerType;
     AudioCodecType_T        codecType;
     AudioDriverType_T       driverType;
@@ -650,6 +779,7 @@ struct SharedFolder
                     strHostPath;
     bool            fWritable;
     bool            fAutoMount;
+    com::Utf8Str    strAutoMountPoint;
 };
 
 typedef std::list<SharedFolder> SharedFoldersList;
@@ -686,11 +816,12 @@ struct CpuIdLeaf
 
     bool operator==(const CpuIdLeaf &c) const;
 
-    uint32_t                ulId;
-    uint32_t                ulEax;
-    uint32_t                ulEbx;
-    uint32_t                ulEcx;
-    uint32_t                ulEdx;
+    uint32_t                idx;
+    uint32_t                idxSub;
+    uint32_t                uEax;
+    uint32_t                uEbx;
+    uint32_t                uEcx;
+    uint32_t                uEdx;
 };
 
 typedef std::list<CpuIdLeaf> CpuIdLeafsList;
@@ -877,7 +1008,6 @@ struct Hardware
     bool areParavirtDefaultSettings(SettingsVersion_T sv) const;
     bool areBootOrderDefaultSettings() const;
     bool areDisplayDefaultSettings() const;
-    bool areVideoCaptureDefaultSettings() const;
     bool areAllNetworkAdaptersDefaultSettings(SettingsVersion_T sv) const;
 
     bool operator==(const Hardware&) const;
@@ -891,6 +1021,7 @@ struct Hardware
                         fVPID,
                         fUnrestrictedExecution,
                         fHardwareVirtForce,
+                        fUseNativeApi,
                         fSyntheticCpu,
                         fTripleFaultReset,
                         fPAE,
@@ -900,6 +1031,11 @@ struct Hardware
     bool                fIBPBOnVMEntry;         //< added out of cycle, after 1.16 was out.
     bool                fSpecCtrl;              //< added out of cycle, after 1.16 was out.
     bool                fSpecCtrlByHost;        //< added out of cycle, after 1.16 was out.
+    bool                fL1DFlushOnSched ;      //< added out of cycle, after 1.16 was out.
+    bool                fL1DFlushOnVMEntry ;    //< added out of cycle, after 1.16 was out.
+    bool                fMDSClearOnSched;       //< added out of cycle, after 1.16 was out.
+    bool                fMDSClearOnVMEntry;     //< added out of cycle, after 1.16 was out.
+    bool                fNestedHWVirt;          //< requires settings version 1.17 (VirtualBox 6.0)
     typedef enum LongModeType { LongMode_Enabled, LongMode_Disabled, LongMode_Legacy } LongModeType;
     LongModeType        enmLongMode;
     uint32_t            cCPUs;
@@ -922,16 +1058,6 @@ struct Hardware
     bool                fAccelerate3D,
                         fAccelerate2DVideo;     // requires settings version 1.8 (VirtualBox 3.1)
 
-    uint32_t            ulVideoCaptureHorzRes;  // requires settings version 1.14 (VirtualBox 4.3)
-    uint32_t            ulVideoCaptureVertRes;  // requires settings version 1.14 (VirtualBox 4.3)
-    uint32_t            ulVideoCaptureRate;     // requires settings version 1.14 (VirtualBox 4.3)
-    uint32_t            ulVideoCaptureFPS;      // requires settings version 1.14 (VirtualBox 4.3)
-    uint32_t            ulVideoCaptureMaxTime;  // requires settings version 1.14 (VirtualBox 4.3)
-    uint32_t            ulVideoCaptureMaxSize;  // requires settings version 1.14 (VirtualBox 4.3)
-    bool                fVideoCaptureEnabled;   // requires settings version 1.14 (VirtualBox 4.3)
-    uint64_t            u64VideoCaptureScreens; // requires settings version 1.14 (VirtualBox 4.3)
-    com::Utf8Str        strVideoCaptureFile;    // requires settings version 1.14 (VirtualBox 4.3)
-
     FirmwareType_T      firmwareType;           // requires settings version 1.9 (VirtualBox 3.1)
 
     PointingHIDType_T   pointingHIDType;        // requires settings version 1.10 (VirtualBox 3.2)
@@ -946,6 +1072,7 @@ struct Hardware
     VRDESettings        vrdeSettings;
 
     BIOSSettings        biosSettings;
+    RecordingSettings   recordingSettings;
     USB                 usbSettings;
     NetworkAdaptersList llNetworkAdapters;
     SerialPortsList     llSerialPorts;
@@ -1161,4 +1288,4 @@ private:
 } // namespace settings
 
 
-#endif /* ___VBox_settings_h */
+#endif /* !VBOX_INCLUDED_settings_h */
