@@ -1,5 +1,10 @@
+/* $Id: ata.c $ */
+/** @file
+ * PC BIOS - ???
+ */
+
 /*
- * Copyright (C) 2006-2016 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -38,6 +43,15 @@
  *
  */
 
+/*
+ * Oracle LGPL Disclaimer: For the avoidance of doubt, except that if any license choice
+ * other than GPL or LGPL is available it will apply instead, Oracle elects to use only
+ * the Lesser General Public License version 2.1 (LGPLv2) at this time for any software where
+ * a choice of LGPL license versions is made available with the language indicating
+ * that LGPLv2 or any later version may be used, or where a choice of which version
+ * of the LGPL is applied is otherwise unspecified.
+ */
+
 
 #include <stdint.h>
 #include <stdarg.h>
@@ -528,25 +542,41 @@ void BIOSCALL ata_detect(void)
             bios_dsk->devices[device].lchs           = lgeo;
             if (device < 2)
             {
-                uint8_t         sum, i;
-                fdpt_t __far    *fdpt;
+                uint8_t             sum, i;
+                fdpt_t __far        *fdpt;
+                void __far * __far  *int_vec;
 
                 if (device == 0)
                     fdpt = ebda_seg :> &EbdaData->fdpt0;
                 else
                     fdpt = ebda_seg :> &EbdaData->fdpt1;
 
+#if 0
+                /* Place the FDPT outside of conventional memory. Needed for
+                 * 286 XENIX 2.1.3/2.2.1 because it completely wipes out
+                 * the EBDA and low memory. Hack!
+                 */
+                fdpt = MK_FP(0xE200, 0xf00);
+                fdpt += device;
+#endif
+
+                /* Set the INT 41h or 46h pointer. */
+                int_vec  = MK_FP(0, (0x41 + device * 5) * sizeof(void __far *));
+                *int_vec = fdpt;
+
                 /* Update the DPT for drive 0/1 pointed to by Int41/46. This used
                  * to be done at POST time with lots of ugly assembler code, which
                  * isn't worth the effort of converting from AMI to Award CMOS
                  * format. Just do it here. */
+                fdpt->resvd1 = fdpt->resvd2 = 0;
+
                 fdpt->lcyl  = lgeo.cylinders;
                 fdpt->lhead = lgeo.heads;
                 fdpt->sig   = 0xa0;
                 fdpt->spt   = spt;
                 fdpt->cyl   = cylinders;
                 fdpt->head  = heads;
-                fdpt->lspt  = spt;
+                fdpt->lspt  = lgeo.spt;
                 sum = 0;
                 for (i = 0; i < 0xf; i++)
                     sum += *((uint8_t __far *)fdpt + i);
