@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2008-2016 Oracle Corporation
+ * Copyright (C) 2008-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,11 +17,12 @@
 
 
 #include <iprt/assert.h>
+#include <iprt/errcore.h>
 #include <iprt/log.h>
 #include <iprt/types.h>
 #include <iprt/string.h>
 #include <VBox/scsi.h>
-#include "ide.h"
+#include <VBox/ata.h>
 
 #ifdef LOG_ENABLED
 
