@@ -1,9 +1,10 @@
+# $Id: routines.sh $
 # Oracle VM VirtualBox
 # VirtualBox installer shell routines
 #
 
 #
-# Copyright (C) 2007-2017 Oracle Corporation
+# Copyright (C) 2007-2019 Oracle Corporation
 #
 # This file is part of VirtualBox Open Source Edition (OSE), as
 # available from http://www.virtualbox.org. This file is free software;
@@ -44,7 +45,7 @@ create_log()
     fi
 }
 
-## Writes text to standard error
+## Writes text to standard error, as standard output is masked.
 #
 # Syntax: info text
 info()
@@ -52,6 +53,14 @@ info()
     echo 1>&2 "$1"
 }
 
+## Copies standard input to standard error, as standard output is masked.
+#
+# Syntax: info text
+catinfo()
+{
+    cat 1>&2
+}
+
 ## Writes text to the log file
 #
 # Syntax: log text
@@ -164,10 +173,15 @@ ExecStop=${script} stop
 [Install]
 WantedBy=multi-user.target
 EOF
-    systemctl daemon-reexec
 }
 
-## Installs a file containing a shell script as an init script
+use_systemd()
+{
+    test ! -f /sbin/init || test -L /sbin/init
+}
+
+## Installs a file containing a shell script as an init script.  Call
+# finish_init_script_install when all scripts have been installed.
 install_init_script()
 {
     self="install_init_script"
@@ -182,7 +196,7 @@ install_init_script()
     test -L "/sbin/rc${name}" && rm "/sbin/rc${name}"
     ln -s "${script}" "/sbin/rc${name}"
     if test -x "`which systemctl 2>/dev/null`"; then
-        if ! test -f /sbin/init || ls -l /sbin/init | grep -q ">.*systemd"; then
+        if use_systemd; then
             { systemd_wrap_init_script "$script" "$name"; return; }
         fi
     fi
@@ -212,6 +226,16 @@ remove_init_script()
     rm -f "/etc/init.d/$name"
 }
 
+## Tell systemd services have been installed or removed.  Should not be done
+# after each individual one, as systemd can crash if it is done too often
+# (reported by the OL team for OL 7.6, may not apply to other versions.)
+finish_init_script_install()
+{
+    if use_systemd; then
+        systemctl daemon-reload
+    fi
+}
+
 ## Did we install a systemd service?
 systemd_service_installed()
 {
