@@ -1,12 +1,10 @@
 /* $Id: HostNetworkInterfaceImpl.cpp $ */
-
 /** @file
- *
  * VirtualBox COM class implementation
  */
 
 /*
- * Copyright (C) 2006-2016 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,14 +14,16 @@
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
+
+#define LOG_GROUP LOG_GROUP_MAIN_HOSTNETWORKINTERFACE
 #include "HostNetworkInterfaceImpl.h"
 #include "AutoCaller.h"
-#include "Logging.h"
 #include "netif.h"
 #ifdef VBOX_WITH_RESOURCE_USAGE_API
 # include "Performance.h"
 # include "PerformanceImpl.h"
 #endif
+#include "LoggingNew.h"
 
 #include <iprt/cpp/utils.h>
 
@@ -64,7 +64,9 @@ void HostNetworkInterface::FinalRelease()
  *
  * @returns COM result indicator
  * @param   aInterfaceName name of the network interface
- * @param   aGuid GUID of the host network interface
+ * @param   aShortName  short name of the network interface
+ * @param   aGuid       GUID of the host network interface
+ * @param   ifType      interface type
  */
 HRESULT HostNetworkInterface::init(Bstr aInterfaceName, Bstr aShortName, Guid aGuid, HostNetworkInterfaceType_T ifType)
 {
@@ -154,6 +156,90 @@ void HostNetworkInterface::i_unregisterMetrics(PerformanceCollector *aCollector,
 
 #ifdef VBOX_WITH_HOSTNETIF_API
 
+#if defined(RT_OS_WINDOWS)
+HRESULT HostNetworkInterface::saveAdapterConfigParameter(const char *szParamName, const Utf8Str &strValue)
+{
+    AssertReturn(mVirtualBox != NULL, E_POINTER);
+    return mVirtualBox->SetExtraData(Bstr(Utf8StrFmt("HostOnly/{%RTuuid}/%s", mGuid, szParamName)).raw(), Bstr(strValue).raw());
+}
+
+HRESULT HostNetworkInterface::eraseAdapterConfigParameter(const char *szParamName)
+{
+    AssertReturn(mVirtualBox != NULL, E_POINTER);
+    return mVirtualBox->SetExtraData(Bstr(Utf8StrFmt("HostOnly/{%RTuuid}/%s", mGuid, szParamName)).raw(), NULL);
+}
+
+HRESULT HostNetworkInterface::saveAdapterConfigIPv4Dhcp()
+{
+    HRESULT hrc = saveAdapterConfigParameter("IPAddress", "DHCP");
+    if (hrc == S_OK)
+        hrc = eraseAdapterConfigParameter("IPNetMask");
+    return hrc;
+}
+
+HRESULT HostNetworkInterface::saveAdapterConfigIPv4(ULONG addr, ULONG mask)
+{
+    HRESULT hrc = saveAdapterConfigParameter("IPAddress", Utf8StrFmt("%RTnaipv4", addr));
+    if (hrc == S_OK)
+        hrc = saveAdapterConfigParameter("IPNetMask", Utf8StrFmt("%RTnaipv4", mask));
+    return hrc;
+}
+
+HRESULT HostNetworkInterface::saveAdapterConfigIPv6(const Utf8Str& addr, ULONG prefix)
+{
+    HRESULT hrc = saveAdapterConfigParameter("IPV6Address", addr);
+    if (hrc == S_OK)
+        hrc = saveAdapterConfigParameter("IPV6PrefixLen", Utf8StrFmt("%u", prefix));
+    return hrc;
+}
+
+bool HostNetworkInterface::isInConfigFile(void)
+{
+    /* We care about host-only adapters only */
+    if (mIfType != HostNetworkInterfaceType_HostOnly)
+        return true;
+
+    Assert(mVirtualBox != NULL);
+    if (mVirtualBox == NULL)
+        return false; /* Trigger config update, which will fail with proper return code */
+    Bstr tmpName;
+    mVirtualBox->GetExtraData(Bstr(Utf8StrFmt("HostOnly/{%RTuuid}/Name", mGuid)).raw(), tmpName.asOutParam());
+    return (tmpName.isNotEmpty() && tmpName == mInterfaceName);
+
+}
+
+HRESULT HostNetworkInterface::saveAdapterConfig(void)
+{
+    /* We care about host-only adapters only */
+    if (mIfType != HostNetworkInterfaceType_HostOnly)
+        return true;
+
+    HRESULT hrc = saveAdapterConfigParameter("Name", mInterfaceName.raw());
+    if (FAILED(hrc))
+        return hrc;
+    if (m.dhcpEnabled)
+        hrc = saveAdapterConfigIPv4Dhcp();
+    else
+        hrc = saveAdapterConfigIPv4(m.IPAddress, m.networkMask);
+    if (SUCCEEDED(hrc))
+        hrc = saveAdapterConfigIPv6(m.IPV6Address.raw(), m.IPV6NetworkMaskPrefixLength);
+    return hrc;
+}
+
+HRESULT HostNetworkInterface::i_updatePersistentConfig(void)
+{
+    if (mVirtualBox == NULL)
+        return E_POINTER;
+
+    HRESULT hrc = S_OK;
+    if (!isInConfigFile())
+    {
+        hrc = saveAdapterConfig();
+    }
+    return hrc;
+}
+#endif /* defined(RT_OS_WINDOWS) */
+
 HRESULT HostNetworkInterface::updateConfig()
 {
     NETIFINFO info;
@@ -164,19 +250,20 @@ HRESULT HostNetworkInterface::updateConfig()
 
         m.realIPAddress = m.IPAddress = info.IPAddress.u;
         m.realNetworkMask = m.networkMask = info.IPNetMask.u;
-        m.dhcpEnabled = info.bDhcpEnabled;
-        m.realIPV6Address = m.IPV6Address = composeIPv6Address(&info.IPv6Address);
+        m.dhcpEnabled = info.fDhcpEnabled;
+        if (info.IPv6Address.s.Lo || info.IPv6Address.s.Hi)
+            m.realIPV6Address = m.IPV6Address = Bstr(Utf8StrFmt("%RTnaipv6", &info.IPv6Address));
+        else
+            m.realIPV6Address = m.IPV6Address = Bstr("");
         RTNetMaskToPrefixIPv6(&info.IPv6NetMask, &iPrefixIPv6);
         m.realIPV6PrefixLength = m.IPV6NetworkMaskPrefixLength = iPrefixIPv6;
-        m.hardwareAddress = composeHardwareAddress(&info.MACAddress);
-#ifdef RT_OS_WINDOWS
-        m.mediumType = (HostNetworkInterfaceMediumType)info.enmMediumType;
-        m.status = (HostNetworkInterfaceStatus)info.enmStatus;
-#else /* !RT_OS_WINDOWS */
-        m.mediumType = info.enmMediumType;
-        m.status = info.enmStatus;
-#endif /* !RT_OS_WINDOWS */
+        m.hardwareAddress = Bstr(Utf8StrFmt("%RTmac", &info.MACAddress));
+        AssertCompile((unsigned)NETIF_T_UNKNOWN == (unsigned)HostNetworkInterfaceMediumType_Unknown);
+        m.mediumType = (HostNetworkInterfaceMediumType_T)info.enmMediumType;
+        AssertCompile((unsigned)NETIF_S_UNKNOWN == (unsigned)HostNetworkInterfaceStatus_Unknown);
+        m.status = (HostNetworkInterfaceStatus_T)info.enmStatus;
         m.speedMbits = info.uSpeedMbits;
+        m.wireless = info.fWireless;
         return S_OK;
     }
     return rc == VERR_NOT_IMPLEMENTED ? E_NOTIMPL : E_FAIL;
@@ -224,19 +311,20 @@ HRESULT HostNetworkInterface::init(Bstr aInterfaceName, HostNetworkInterfaceType
 
     m.realIPAddress = m.IPAddress = pIf->IPAddress.u;
     m.realNetworkMask = m.networkMask = pIf->IPNetMask.u;
-    m.realIPV6Address = m.IPV6Address = composeIPv6Address(&pIf->IPv6Address);
+    if (pIf->IPv6Address.s.Lo || pIf->IPv6Address.s.Hi)
+        m.realIPV6Address = m.IPV6Address = Bstr(Utf8StrFmt("%RTnaipv6", &pIf->IPv6Address));
+    else
+        m.realIPV6Address = m.IPV6Address = Bstr("");
     RTNetMaskToPrefixIPv6(&pIf->IPv6NetMask, &iPrefixIPv6);
     m.realIPV6PrefixLength = m.IPV6NetworkMaskPrefixLength = iPrefixIPv6;
-    m.dhcpEnabled = pIf->bDhcpEnabled;
-    m.hardwareAddress = composeHardwareAddress(&pIf->MACAddress);
-#ifdef RT_OS_WINDOWS
-    m.mediumType = (HostNetworkInterfaceMediumType)pIf->enmMediumType;
-    m.status = (HostNetworkInterfaceStatus)pIf->enmStatus;
-#else /* !RT_OS_WINDOWS */
-    m.mediumType = pIf->enmMediumType;
-    m.status = pIf->enmStatus;
-#endif /* !RT_OS_WINDOWS */
+    m.dhcpEnabled = pIf->fDhcpEnabled;
+    m.hardwareAddress = Bstr(Utf8StrFmt("%RTmac", &pIf->MACAddress));
+    AssertCompile((unsigned)NETIF_T_UNKNOWN == (unsigned)HostNetworkInterfaceMediumType_Unknown);
+    m.mediumType = (HostNetworkInterfaceMediumType_T)pIf->enmMediumType;
+    AssertCompile((unsigned)NETIF_S_UNKNOWN == (unsigned)HostNetworkInterfaceStatus_Unknown);
+    m.status = (HostNetworkInterfaceStatus_T)pIf->enmStatus;
     m.speedMbits = pIf->uSpeedMbits;
+    m.wireless = pIf->fWireless;
 
     /* Confirm a successful initialization */
     autoInitSpan.setSucceeded();
@@ -278,11 +366,11 @@ HRESULT HostNetworkInterface::getShortName(com::Utf8Str &aShortName)
  * Returns the GUID of the host network interface.
  *
  * @returns COM status code
- * @param   aGuid GUI Id
+ * @param   aGuid GUID
  */
-HRESULT HostNetworkInterface::getId(com::Guid &aGuiId)
+HRESULT HostNetworkInterface::getId(com::Guid &aGuid)
 {
-    aGuiId = mGuid;
+    aGuid = mGuid;
 
     return S_OK;
 }
@@ -368,10 +456,10 @@ HRESULT HostNetworkInterface::getIPV6Address(com::Utf8Str &aIPV6Address)
 }
 
 /**
- * Returns the IP V6 network mask of the host network interface.
+ * Returns the IP V6 network mask prefix length of the host network interface.
  *
  * @returns COM status code
- * @param   aIPV6Mask address of result pointer
+ * @param   aIPV6NetworkMaskPrefixLength address of result pointer
  */
 HRESULT HostNetworkInterface::getIPV6NetworkMaskPrefixLength(ULONG *aIPV6NetworkMaskPrefixLength)
 {
@@ -439,6 +527,13 @@ HRESULT HostNetworkInterface::getNetworkName(com::Utf8Str &aNetworkName)
     return S_OK;
 }
 
+HRESULT HostNetworkInterface::getWireless(BOOL *aWireless)
+{
+    *aWireless = m.wireless;
+
+    return S_OK;
+}
+
 HRESULT HostNetworkInterface::enableStaticIPConfig(const com::Utf8Str &aIPAddress,
                                                    const com::Utf8Str &aNetworkMask)
 {
@@ -453,12 +548,17 @@ HRESULT HostNetworkInterface::enableStaticIPConfig(const com::Utf8Str &aIPAddres
             if (RT_SUCCESS(rc))
             {
                 m.realIPAddress = 0;
+#if defined(RT_OS_WINDOWS)
+                eraseAdapterConfigParameter("IPAddress");
+                eraseAdapterConfigParameter("IPNetMask");
+#else /* !defined(RT_OS_WINDOWS) */
                 if (FAILED(mVirtualBox->SetExtraData(BstrFmt("HostOnly/%ls/IPAddress",
                                                              mInterfaceName.raw()).raw(), NULL)))
                     return E_FAIL;
                 if (FAILED(mVirtualBox->SetExtraData(BstrFmt("HostOnly/%ls/IPNetMask",
                                                              mInterfaceName.raw()).raw(), NULL)))
                     return E_FAIL;
+#endif /* !defined(RT_OS_WINDOWS) */
                 return S_OK;
             }
         }
@@ -483,6 +583,9 @@ HRESULT HostNetworkInterface::enableStaticIPConfig(const com::Utf8Str &aIPAddres
             {
                 m.realIPAddress   = ip;
                 m.realNetworkMask = mask;
+#if defined(RT_OS_WINDOWS)
+                saveAdapterConfigIPv4(ip, mask);
+#else /* !defined(RT_OS_WINDOWS) */
                 if (FAILED(mVirtualBox->SetExtraData(BstrFmt("HostOnly/%ls/IPAddress",
                                                              mInterfaceName.raw()).raw(),
                                                      Bstr(aIPAddress).raw())))
@@ -491,6 +594,7 @@ HRESULT HostNetworkInterface::enableStaticIPConfig(const com::Utf8Str &aIPAddres
                                                              mInterfaceName.raw()).raw(),
                                                      Bstr(aNetworkMask).raw())))
                     return E_FAIL;
+#endif /* !defined(RT_OS_WINDOWS) */
                 return S_OK;
             }
             else
@@ -512,10 +616,32 @@ HRESULT HostNetworkInterface::enableStaticIPConfigV6(const com::Utf8Str &aIPV6Ad
     return E_NOTIMPL;
 #else
     if (aIPV6NetworkMaskPrefixLength > 128)
-        return E_INVALIDARG;
+        return mVirtualBox->setErrorBoth(E_INVALIDARG, VERR_INVALID_PARAMETER,
+                   "Invalid IPv6 prefix length");
 
-    int rc = S_OK;
-    if (   m.realIPV6Address      != aIPV6Address
+    int rc;
+
+    RTNETADDRIPV6 AddrOld, AddrNew;
+    char *pszZoneIgnored;
+    bool fAddrChanged;
+
+    rc = RTNetStrToIPv6Addr(aIPV6Address.c_str(), &AddrNew, &pszZoneIgnored);
+    if (RT_FAILURE(rc))
+    {
+        return mVirtualBox->setErrorBoth(E_INVALIDARG, rc, "Invalid IPv6 address");
+    }
+
+    rc = RTNetStrToIPv6Addr(com::Utf8Str(m.realIPV6Address).c_str(), &AddrOld, &pszZoneIgnored);
+    if (RT_SUCCESS(rc))
+    {
+        fAddrChanged = (AddrNew.s.Lo != AddrOld.s.Lo || AddrNew.s.Hi != AddrOld.s.Hi);
+    }
+    else
+    {
+        fAddrChanged = true;
+    }
+
+    if (   fAddrChanged
         || m.realIPV6PrefixLength != aIPV6NetworkMaskPrefixLength)
     {
         BSTR bstr;
@@ -533,6 +659,9 @@ HRESULT HostNetworkInterface::enableStaticIPConfigV6(const com::Utf8Str &aIPV6Ad
         {
             m.realIPV6Address = aIPV6Address;
             m.realIPV6PrefixLength = aIPV6NetworkMaskPrefixLength;
+#if defined(RT_OS_WINDOWS)
+            saveAdapterConfigIPv6(Bstr(aIPV6Address).raw(), aIPV6NetworkMaskPrefixLength);
+#else /* !defined(RT_OS_WINDOWS) */
             if (FAILED(mVirtualBox->SetExtraData(BstrFmt("HostOnly/%ls/IPV6Address",
                                                          mInterfaceName.raw()).raw(),
                                                  Bstr(aIPV6Address).raw())))
@@ -540,6 +669,7 @@ HRESULT HostNetworkInterface::enableStaticIPConfigV6(const com::Utf8Str &aIPV6Ad
             if (FAILED(mVirtualBox->SetExtraData(BstrFmt("HostOnly/%ls/IPV6NetMask",
                                                          mInterfaceName.raw()).raw(),
                                                  BstrFmt("%u", aIPV6NetworkMaskPrefixLength).raw())))
+#endif /* !defined(RT_OS_WINDOWS) */
                 return E_FAIL;
         }
 
