@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2013-2017 Oracle Corporation
+ * Copyright (C) 2013-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -2427,7 +2427,7 @@ pxtcp_pcb_pull_inbound(void *ctx)
     }
 
     pxtcp->inbound_pull = 1;
-    if (pxtcp->outbound_close_done) {
+    if (pxtcp->pmhdl.slot < 0) {
         DPRINTF(("%s: pxtcp %p: pcb %p (deferred delete)\n",
                  __func__, (void *)pxtcp, (void *)pxtcp->pcb));
         pxtcp->deferred_delete = 1;
