@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010-2017 Oracle Corporation
+ * Copyright (C) 2010-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -655,16 +655,16 @@ static int rtVfsChainSpecMakeFinalPathElement(PRTVFSCHAINELEMSPEC pElement, cons
  * Finds the end of the argument string.
  *
  * @returns The offset of the end character relative to @a psz.
- * @param   psz                 The argument string.
+ * @param   psz             The argument string.
+ * @param   chCloseParen    The closing parenthesis.
  */
-static size_t rtVfsChainSpecFindArgEnd(const char *psz)
+static size_t rtVfsChainSpecFindArgEnd(const char *psz, char const chCloseParen)
 {
     size_t off = 0;
     char   ch;
     while (  (ch = psz[off]) != '\0'
            && ch != ','
-           && ch != ')'
-           && ch != '}')
+           && ch != chCloseParen)
     {
         if (   ch == '\\'
             && rtVfsChainSpecIsEscapableChar(psz[off+1]))
@@ -777,12 +777,13 @@ RTDECL(int) RTVfsChainSpecParse(const char *pszSpec, uint32_t fFlags, RTVFSOBJTY
                 rc = VERR_VFS_CHAIN_EXPECTED_LEFT_PARENTHESES;
             break;
         }
+        char const chCloseParen = (chOpenParen == '(' ? ')' : '}');
         pszSrc = RTStrStripL(pszSrc + cch + 1);
 
         /*
          * The name of the element provider.
          */
-        cch = rtVfsChainSpecFindArgEnd(pszSrc);
+        cch = rtVfsChainSpecFindArgEnd(pszSrc, chCloseParen);
         if (!cch)
         {
             rc = VERR_VFS_CHAIN_EXPECTED_PROVIDER_NAME;
@@ -799,7 +800,7 @@ RTDECL(int) RTVfsChainSpecParse(const char *pszSpec, uint32_t fFlags, RTVFSOBJTY
         while (*pszSrc == ',')
         {
             pszSrc = RTStrStripL(pszSrc + 1);
-            cch = rtVfsChainSpecFindArgEnd(pszSrc);
+            cch = rtVfsChainSpecFindArgEnd(pszSrc, chCloseParen);
             rc = rtVfsChainSpecElementAddArg(pElement, pszSrc, cch, (uint16_t)(pszSrc - pszSpec));
             if (RT_FAILURE(rc))
                 break;
@@ -809,7 +810,7 @@ RTDECL(int) RTVfsChainSpecParse(const char *pszSpec, uint32_t fFlags, RTVFSOBJTY
             break;
 
         /* Must end with a right parentheses/curly. */
-        if (*pszSrc != (chOpenParen == '(' ? ')' : '}'))
+        if (*pszSrc != chCloseParen)
         {
             rc = VERR_VFS_CHAIN_EXPECTED_RIGHT_PARENTHESES;
             break;
@@ -1113,6 +1114,120 @@ RTDECL(int) RTVfsChainElementDeregisterProvider(PRTVFSCHAINELEMENTREG pRegRec, b
 }
 
 
+RTDECL(int) RTVfsChainOpenObj(const char *pszSpec, uint64_t fFileOpen, uint32_t fObjFlags,
+                              PRTVFSOBJ phVfsObj, uint32_t *poffError, PRTERRINFO pErrInfo)
+{
+    /*
+     * Validate input.
+     */
+    uint32_t offErrorIgn;
+    if (!poffError)
+        poffError = &offErrorIgn;
+    *poffError = 0;
+    AssertPtrReturn(pszSpec, VERR_INVALID_POINTER);
+    AssertReturn(*pszSpec != '\0', VERR_INVALID_PARAMETER);
+    AssertPtrReturn(phVfsObj, VERR_INVALID_POINTER);
+    AssertPtrNullReturn(pErrInfo, VERR_INVALID_POINTER);
+
+    int rc = rtFileRecalcAndValidateFlags(&fFileOpen);
+    if (RT_FAILURE(rc))
+        return rc;
+    AssertMsgReturn(   RTPATH_F_IS_VALID(fObjFlags, RTVFSOBJ_F_VALID_MASK)
+                    && (fObjFlags & RTVFSOBJ_F_CREATE_MASK) <= RTVFSOBJ_F_CREATE_DIRECTORY,
+                    ("fObjFlags=%#x\n", fObjFlags),
+                    VERR_INVALID_FLAGS);
+
+    /*
+     * Try for a VFS chain first, falling back on regular file system stuff if it's just a path.
+     */
+    PRTVFSCHAINSPEC pSpec = NULL;
+    if (strncmp(pszSpec, RTVFSCHAIN_SPEC_PREFIX, sizeof(RTVFSCHAIN_SPEC_PREFIX) - 1) == 0)
+    {
+        rc = RTVfsChainSpecParse(pszSpec,  0 /*fFlags*/, RTVFSOBJTYPE_DIR, &pSpec, poffError);
+        if (RT_FAILURE(rc))
+            return rc;
+
+        Assert(pSpec->cElements > 0);
+        if (   pSpec->cElements > 1
+            || pSpec->paElements[0].enmType != RTVFSOBJTYPE_END)
+        {
+            const char *pszFinal = NULL;
+            RTVFSOBJ    hVfsObj  = NIL_RTVFSOBJ;
+            pSpec->fOpenFile = fFileOpen;
+            rc = RTVfsChainSpecCheckAndSetup(pSpec, NULL /*pReuseSpec*/, &hVfsObj, &pszFinal, poffError, pErrInfo);
+            if (RT_SUCCESS(rc))
+            {
+                if (!pszFinal)
+                {
+                    *phVfsObj = hVfsObj;
+                    rc = VINF_SUCCESS;
+                }
+                else
+                {
+                    /*
+                     * Do a file open with the final path on the returned object.
+                     */
+                    RTVFS           hVfs    = RTVfsObjToVfs(hVfsObj);
+                    RTVFSDIR        hVfsDir = RTVfsObjToDir(hVfsObj);
+                    RTVFSFSSTREAM   hVfsFss = RTVfsObjToFsStream(hVfsObj);
+                    if (hVfs != NIL_RTVFS)
+                        rc = RTVfsObjOpen(hVfs, pszFinal, fFileOpen, fObjFlags, phVfsObj);
+                    else if (hVfsDir != NIL_RTVFSDIR)
+                        rc = RTVfsDirOpenObj(hVfsDir, pszFinal, fFileOpen, fObjFlags, phVfsObj);
+                    else if (hVfsFss != NIL_RTVFSFSSTREAM)
+                        rc = VERR_NOT_IMPLEMENTED;
+                    else
+                        rc = VERR_VFS_CHAIN_TYPE_MISMATCH_PATH_ONLY;
+                    RTVfsRelease(hVfs);
+                    RTVfsDirRelease(hVfsDir);
+                    RTVfsFsStrmRelease(hVfsFss);
+                    RTVfsObjRelease(hVfsObj);
+                }
+            }
+
+            RTVfsChainSpecFree(pSpec);
+            return rc;
+        }
+
+        /* Only a path element. */
+        pszSpec = pSpec->paElements[0].paArgs[0].psz;
+    }
+
+    /*
+     * Path to regular file system.
+     * Go via the directory VFS wrapper to avoid duplicating code.
+     */
+    RTVFSDIR hVfsParentDir = NIL_RTVFSDIR;
+    const char *pszFilename;
+    if (RTPathHasPath(pszSpec))
+    {
+        char *pszCopy = RTStrDup(pszSpec);
+        if (pszCopy)
+        {
+            RTPathStripFilename(pszCopy);
+            rc = RTVfsDirOpenNormal(pszCopy, 0 /*fOpen*/, &hVfsParentDir);
+            RTStrFree(pszCopy);
+        }
+        else
+            rc = VERR_NO_STR_MEMORY;
+        pszFilename = RTPathFilename(pszSpec);
+    }
+    else
+    {
+        pszFilename = pszSpec;
+        rc = RTVfsDirOpenNormal(".", 0 /*fOpen*/, &hVfsParentDir);
+    }
+    if (RT_SUCCESS(rc))
+    {
+        rc = RTVfsDirOpenObj(hVfsParentDir, pszFilename, fFileOpen, fObjFlags, phVfsObj);
+        RTVfsDirRelease(hVfsParentDir);
+    }
+
+    RTVfsChainSpecFree(pSpec);
+    return rc;
+}
+
+
 RTDECL(int) RTVfsChainOpenDir(const char *pszSpec, uint32_t fOpen,
                               PRTVFSDIR phVfsDir, uint32_t *poffError, PRTERRINFO pErrInfo)
 {
@@ -1140,16 +1255,15 @@ RTDECL(int) RTVfsChainOpenDir(const char *pszSpec, uint32_t fOpen,
         if (   pSpec->cElements > 1
             || pSpec->paElements[0].enmType != RTVFSOBJTYPE_END)
         {
-
             const char *pszFinal = NULL;
             RTVFSOBJ    hVfsObj  = NIL_RTVFSOBJ;
-            pSpec->fOpenFile = fOpen;
+            pSpec->fOpenFile = RTFILE_O_READ;
             rc = RTVfsChainSpecCheckAndSetup(pSpec, NULL /*pReuseSpec*/, &hVfsObj, &pszFinal, poffError, pErrInfo);
             if (RT_SUCCESS(rc))
             {
                 if (!pszFinal)
                 {
-                    /* Try convert it to a file object and we're done. */
+                    /* Try convert it to a directory object and we're done. */
                     *phVfsDir = RTVfsObjToDir(hVfsObj);
                     if (*phVfsDir)
                         rc = VINF_SUCCESS;
@@ -1194,7 +1308,6 @@ RTDECL(int) RTVfsChainOpenDir(const char *pszSpec, uint32_t fOpen,
 
     RTVfsChainSpecFree(pSpec);
     return rc;
-
 }
 
 
@@ -1629,7 +1742,7 @@ RTDECL(int) RTVfsChainQueryInfo(const char *pszSpec, PRTFSOBJINFO pObjInfo, RTFS
 RTDECL(bool) RTVfsChainIsSpec(const char *pszSpec)
 {
     return pszSpec
-        && strcmp(pszSpec, RTVFSCHAIN_SPEC_PREFIX) == 0;
+        && strncmp(pszSpec, RT_STR_TUPLE(RTVFSCHAIN_SPEC_PREFIX)) == 0;
 }
 
 
